@@ -11,6 +11,7 @@ import timeit
 import logging
 import warnings
 from datetime import datetime
+from copy import deepcopy
 try:  # pragma: no cover
     from collections import abc
 except ImportError:  # pragma: no cover
@@ -66,22 +67,25 @@ class DefaultFdMixin(ProgressBarMixinBase):
             fd = utils.streams.original_stderr
 
         self.fd = fd
+        self.is_ansi_terminal = utils.is_ansi_terminal(fd)
 
         # Check if this is an interactive terminal
-        self.is_terminal = is_terminal = utils.is_terminal(fd, is_terminal)
+        self.is_terminal = utils.is_terminal(
+            fd, is_terminal or self.is_ansi_terminal)
 
         # Check if it should overwrite the current line (suitable for
         # iteractive terminals) or write line breaks (suitable for log files)
         if line_breaks is None:
             line_breaks = utils.env_flag('PROGRESSBAR_LINE_BREAKS', not
-                                         is_terminal)
+                                         self.is_terminal)
         self.line_breaks = line_breaks
 
         # Check if ANSI escape characters are enabled (suitable for iteractive
         # terminals), or should be stripped off (suitable for log files)
         if enable_colors is None:
             enable_colors = utils.env_flag('PROGRESSBAR_ENABLE_COLORS',
-                                           is_terminal)
+                                           self.is_ansi_terminal)
+
         self.enable_colors = enable_colors
 
         ProgressBarMixinBase.__init__(self, **kwargs)
@@ -98,7 +102,10 @@ class DefaultFdMixin(ProgressBarMixinBase):
         else:
             line = '\r' + line
 
-        self.fd.write(line)
+        try:  # pragma: no cover
+            self.fd.write(line)
+        except UnicodeEncodeError:  # pragma: no cover
+            self.fd.write(line.encode('ascii', 'replace'))
 
     def finish(self, *args, **kwargs):  # pragma: no cover
         if self._finished:
@@ -301,7 +308,7 @@ class ProgressBar(StdRedirectMixin, ResizableMixin, ProgressBarBase):
         self.min_value = min_value
         self.max_value = max_value
         self.max_error = max_error
-        self.widgets = widgets
+        self.widgets = deepcopy(widgets)
         self.prefix = prefix
         self.suffix = suffix
         self.widget_kwargs = widget_kwargs or {}
@@ -533,9 +540,12 @@ class ProgressBar(StdRedirectMixin, ResizableMixin, ProgressBarBase):
         except StopIteration:
             self.finish()
             raise
+        except GeneratorExit:  # pragma: no cover
+            self.finish(dirty=True)
+            raise
 
     def __exit__(self, exc_type, exc_value, traceback):
-        self.finish()
+        self.finish(dirty=bool(exc_type))
 
     def __enter__(self):
         return self
